
// Include Libraries
#include <esp_now.h>
#include <esp_wifi.h>
#include <WiFi.h>

#include <Adafruit_NeoPixel.h>

#define PIN 7  // ESP32-C3 built-in RGB led
#define NUMPIXELS 1

Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);
#define DELAYVAL 134

// Variables for test data
int int_value;
float float_value;
bool bool_value = true;

// MAC Address of responder - edit as required
//uint8_t broadcastAddress[] = {0x24, 0x6F, 0x28, 0x7A, 0xAE, 0x7C};
//uint8_t broadcastAddress[] = {0xA0, 0x76, 0x4E, 0x1B, 0x99, 0xF4};
uint8_t broadcastAddress[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

// Define a data structure
typedef struct struct_message {
  char a[32];
  int b;
  float c;
  bool d;
} struct_message;

// Create a structured object
struct_message myData;

float proximity;

// Peer info
esp_now_peer_info_t peerInfo;

typedef struct {
  unsigned frame_ctrl:16;
  unsigned duration_id:16;
  uint8_t addr1[6]; /* receiver address */
  uint8_t addr2[6]; /* sender address */
  uint8_t addr3[6]; /* filtering address */
  unsigned sequence_ctrl:16;
  uint8_t addr4[6]; /* optional */
} wifi_ieee80211_mac_hdr_t;

typedef struct {
  wifi_ieee80211_mac_hdr_t hdr;
  uint8_t payload[0]; /* network data ended with 4 bytes csum (CRC32) */
} wifi_ieee80211_packet_t;

void promiscuous_rx_cb(void *buf, wifi_promiscuous_pkt_type_t type) {

    // All espnow traffic uses action frames which are a subtype of the mgmnt frames so filter out everything else.
    if (type != WIFI_PKT_MGMT)
        return;

    static const uint8_t ACTION_SUBTYPE = 0xd0;
    //static const uint8_t ESPRESSIF_OUI[] = {0x18, 0xfe, 0x34};
    static const uint8_t ESPRESSIF_OUI[] = {0xA0, 0x76, 0x4E};

    const wifi_promiscuous_pkt_t *ppkt = (wifi_promiscuous_pkt_t *)buf;
    const wifi_ieee80211_packet_t *ipkt = (wifi_ieee80211_packet_t *)ppkt->payload;
    const wifi_ieee80211_mac_hdr_t *hdr = &ipkt->hdr;



    // Only continue processing if this is an action frame containing the Espressif OUI.
    if ((ACTION_SUBTYPE == (hdr->frame_ctrl & 0xFF)) && 
        //(memcmp(hdr->addr1, ESPRESSIF_OUI, 3) == 0) &&
        (memcmp(hdr->addr2, ESPRESSIF_OUI, 3) == 0)) {
         int rssi = ppkt->rx_ctrl.rssi;
         //uint8_t addr1 = hdr->addr1[0];
         //uint8_t addr2 = hdr->addr2[0];
         //uint8_t addr3 = hdr->addr3[0];
         //uint8_t addr4 = hdr->addr4[0];
         //Serial.println(rssi);
         //Serial.println(hdr->addr1[5]);
         //Serial.println(hdr->addr2[5]);
         //Serial.println(addr3);
         //Serial.println(addr4);
         proximity = rssi;
     }
}

// Callback function called when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");

  /*
  int scanResults = WiFi.scanNetworks();

  for (int i = 0; i < scanResults; ++i) {

    String SSID = WiFi.SSID(i);
    int32_t RSSI = WiFi.RSSI(i);
    String BSSIDstr = WiFi.BSSIDstr(i);
    Serial.print(i + 1); Serial.print(": "); Serial.print(SSID); Serial.print(" ["); Serial.print(BSSIDstr); Serial.print("]"); Serial.print(" ("); Serial.print(RSSI); Serial.print(")"); Serial.println("");
  }
  */

  if (status == ESP_NOW_SEND_SUCCESS) {
    //int32_t RSSI = WiFi.RSSI(1);
    Serial.println("Delivery Success ");
    //Serial.print("Signal: ");
    //Serial.print(RSSI);
  } else {
    Serial.println("Delivery Fail");
  }

}

// Callback function executed when data is received
void OnDataRecv(const uint8_t * mac, const uint8_t *incomingData, int len) {
  memcpy(&myData, incomingData, sizeof(myData));
  //Serial.print("Data received: ");
  //Serial.println(len);
  //Serial.print("Character Value: ");
  //Serial.println(myData.a);
  //Serial.print("Integer Value: ");
  //Serial.println(myData.b);
  //Serial.print("Float Value: ");
  //Serial.println(myData.c);
  //Serial.print("Boolean Value: ");
  //Serial.println(myData.d);
  //Serial.println();
}

void setup() {
  
  // Set up Serial Monitor
  Serial.begin(115200);
 
  // Set ESP32 as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  pixels.begin();

  //WiFi.setSleep(false);

  // Initilize ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // Register the send callback
  esp_now_register_send_cb(OnDataSent);
  esp_wifi_set_promiscuous(true);
  esp_wifi_set_promiscuous_rx_cb(&promiscuous_rx_cb);
  
  // Register peer
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;
  
  //esp_now_register_recv_cb(OnDataRecv);

  /*
  int32_t RSSI = WiFi.RSSI(0);
  Serial.print("Delivery Success ");
  Serial.print("Signal: ");
  Serial.print(RSSI);
  */

  // Add peer        
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    //Serial.println("Failed to add peer");
    return;
  }
}

void loop() {

  // Create test data

  // Generate a random integer
  int_value = random(1,20);

  // Use integer to make a new float
  float_value = 1.3 * int_value;

  // Invert the boolean value
  bool_value = !bool_value;
  
  // Format structured data
  strcpy(myData.a, "Welcome to Resilient Wearables");
  myData.b = int_value;
  myData.c = float_value;
  myData.d = bool_value;
  


  // Send message via ESP-NOW
  esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &myData, sizeof(myData));
   
  if (result == ESP_OK) {
    //Serial.println("Sending confirmed");
    pixels.clear();
    Serial.println(proximity);
    //pixels.setPixelColor(0, pixels.Color(10, abs(proximity*3), 10));
    //int prox = 220 - (abs(proximity)*3);
    //float prox = (255 * (proximity-255)/(0-(-70)));
    int output_start = 0;
    int output_end = 255;
    int input_start = 70;
    int input_end = 1; 
    int input = abs(proximity);
    if ( input > input_start ) {
      input = input_start;
    }

    double slope = 1.0 * (output_end - output_start) / (input_end - input_start );
    float prox = output_start + round(slope * (input - input_start));
    Serial.println(prox);
    pixels.setPixelColor(0, pixels.Color(0, prox, 0));
    pixels.show();
  }
  else {
    Serial.println("Sending error");
  }
  delay(random(5,20)*100);
}